#!/bin/bash
#SBATCH --time=2:22:22                   # Elapsed time limit in hh:mm:ss
#SBATCH -N1 -n1                          # Number of nodes (N) and MPI tasks (n)
#SBATCH --error std_err_run9.txt         # Error output
#SBATCH --partition=skl_fua_prod         # Choosing Skylake (A3) Eurofusion production nodes
#SBATCH --account=FUA32_Selavlas         # Project ID
#SBATCH --job-name=run9_2G2d3v           # Request name

PICVERT_HOME=$HOME/Pic-Vert
FOLDER=$PICVERT_HOME/scripts_marconi/2d3v_performance

export LD_LIBRARY_PATH=/cineca/prod/opt/libraries/fftw/3.3.4/intelmpi--2017--binary/lib:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH=/cineca/prod/opt/libraries/szip/2.1/gnu--6.1.0/lib:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH=/cineca/prod/opt/libraries/zlib/1.2.8/gnu--6.1.0/lib:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH=/cineca/prod/opt/libraries/hdf5/1.8.17/intelmpi--2017--binary/lib:$LD_LIBRARY_PATH
module load intel intelmpi

run() {
  export OMP_NUM_THREADS=$1
  cd $FOLDER/run$2
  export KMP_AFFINITY=granularity=fine,compact,1,0,verbose
  srun --mpi=pmi2 ./aocosoa_2d3v.out $FOLDER/run$2/parameters_2d3v.txt
}

#For some reason, setting "#SBATCH --output std_24thread.txt" does not work (some printfs get lost).
run_id=9
std_output="${FOLDER}/std_output_run${run_id}.txt"
run 24 $run_id > "${std_output}"

echo ""                                       >> "${std_output}"
echo -n "The job id was "                     >> "${std_output}"
echo $SLURM_JOB_ID                            >> "${std_output}"
echo -n "The nodes assigned to the job were " >> "${std_output}"
echo $SLURM_JOB_NODELIST                      >> "${std_output}"
