#!/bin/bash
#SBATCH --time=4:44:44                   # Elapsed time limit in hh:mm:ss
#SBATCH -N1 -n1                          # Number of nodes (N) and MPI tasks (n)
#SBATCH --error std_err.txt              # Error output
#SBATCH --partition=skl_fua_prod         # Choosing Skylake (A3) Eurofusion production nodes
#SBATCH --account=FUA32_Selavlas         # Project ID
#SBATCH --job-name=1G3d_atomics          # Request name

PICVERT_HOME=$HOME/Pic-Vert
FOLDER=$PICVERT_HOME/scripts_marconi/performance_bimaxwellians

export LD_LIBRARY_PATH=/cineca/prod/opt/libraries/fftw/3.3.4/intelmpi--2017--binary/lib:$LD_LIBRARY_PATH
module load intel intelmpi

run() {
  export OMP_NUM_THREADS=$1
  cd $FOLDER/proportion$2-drift$3-vth$4
  export KMP_AFFINITY=granularity=fine,compact,1,0,verbose
  srun --mpi=pmi2 ./aocosoa_3d.out
}

#For some reason, setting "#SBATCH --output std_24thread.txt" does not work (some printfs get lost).
std_output=$FOLDER/std_output.txt

echo -n "" > "${std_output}"
run 24 0.00 0.00 0.339  >> "${std_output}"
run 24 0.02 10.0 0.10   >> "${std_output}"
run 24 0.02 11.0 0.126  >> "${std_output}"
run 24 0.02 13.0 0.178  >> "${std_output}"
run 24 0.02 15.0 0.234  >> "${std_output}"
run 24 0.02 17.0 0.287  >> "${std_output}"
run 24 0.02 20.0 0.355  >> "${std_output}"
run 24 0.02 22.0 0.3585 >> "${std_output}"

echo ""                                       >> "${std_output}"
echo -n "The job id was "                     >> "${std_output}"
echo $SLURM_JOB_ID                            >> "${std_output}"
echo -n "The nodes assigned to the job were " >> "${std_output}"
echo $SLURM_JOB_NODELIST                      >> "${std_output}"
